import { Component, ViewChild } from "@angular/core";
import { NavController, Slides, Searchbar, AlertController } from "ionic-angular";
import { Keyboard } from "ionic-native";
import { DetailsView } from "../../defaults/default-details-view";
import { SaleOrder } from "../../models/sale.order";
import { Partner } from "../../models/partner";
import { Product } from "../../models/product";
import { DataProvider } from "../../providers/data-provider";
import { Slots } from "../../utils/slots";

@Component({
    selector: 'page-sale-order-details',
    templateUrl: 'sale-order-details.html',
    providers: [Partner]
})
export class SaleOrderDetailsPage extends DetailsView<SaleOrder> {
    
    search: boolean = false;
    footer: any = {
        show: true,
        back: {
            text: "Atrás",
            show: false
        },
        forward: {
            text: "Siguiente",
            show: false
        }
    }
    customers: Array<Partner>;
    products: Array<Product>;
    lines: Array<{ product: Product, price: number, quantity: number, subtotal: number }>
    total: number;

    @ViewChild("saleSlides")
    slider: Slides;
    @ViewChild("customerSearchInput")
    customerSearchBar: Searchbar;
    @ViewChild("productSearchInput")
    productSearchBar: Searchbar;
  
    constructor(
        public navCtrl: NavController,
        public alertCtrl: AlertController,
        public db: DataProvider,
        public customer: Partner,
    ) {
        super(SaleOrder);

        this.lines = [];
        this.total = 0;

        this.initKeyBoardEvents();
    }

    /**
     *
     */
    initKeyBoardEvents(): void {
        Keyboard.onKeyboardShow().subscribe(() => { 
            this.footer.show = false;
        });
        Keyboard.onKeyboardHide().subscribe(() => { 
            this.footer.show = true;
        });
    }

    /**
     *
     */
    ionViewDidLoad() {
        console.log('Hello SaleOrderDetailsPage Page');
        this.loadCustomers();
        this.loadProducts();
    }

    /**
     *
     */
    loadCustomers() {
        this.events.publish(Slots.APP_LOADING);

        this.db.getAll(DataProvider.DOCS.PARTNER).then((partners: Array<Partner>) => {
            console.log(partners);
            
            this.customers = partners.filter(item => { 
                return item.customer && item.doc_state != "deleted";
            });

            this.events.publish(Slots.APP_LOADED);
        }).catch(e => {
            this.events.publish(Slots.APP_LOADED);
        });
    }

    /**
     *
     */
    loadProducts() {
        this.db.getAll(DataProvider.DOCS.PRODUCT_TEMPLATE).then((products: Array<Product>) => {
            this.products = products.filter(item => { 
                return item.doc_state != "deleted";
            });
        }).catch(e => {
            console.log(e);
            
        });
    }

    /**
     *
     */
    onSlideChanged(e: Slides): void {
        this.search = false;
        this.customerSearchBar.value = "";
        this.productSearchBar.value = "";
    }

    /**
     *
     */
    goToNext(): void {
        this.slider.slideNext();
        this.footer.back.show = true;

        if (this.slider.getActiveIndex() == this.slider.length() - 1) {
            this.footer.forward.show = false;
        }
    }

    /**
     *
     */
    goToBack(): void {
        this.slider.slidePrev();
        this.footer.forward.show = true;

        if (this.slider.getActiveIndex() == 0) {
            this.footer.back.show = false;
        }
    }

    /**
     *
     */
    showList(): void {
        this.search = !this.search;
    }

    isSearch(): boolean {
        return this.search;
    }

    /**
     *
     */
    searchCustomer(e: any): void {
        let value = e.target.value;

        if (value && value.trim() != "") {
            this.search = true;
            
            return;
        }

        this.search = false;
        this.footer.forward.show = false;
    }

    /**
     *
     */
    selectCustomer(item: Partner): void {
        this.customer = item;
        this.search = false;
        this.customerSearchBar.value = "";
        this.footer.forward.show = true;
    }

    /**
     *
     */
    searchProduct(e: any): void {
        let value = e.target.value;

        if (value && value.trim() != "") {
            this.search = true;
            
            return;
        }
        
        this.search = false;
    }

    /**
     *
     */
    readBarcode(): void {
        console.log("read");
        
    }

    /**
     *
     */
    selectProduct(item: Product): void {
        this.addItem(item);

        this.search = false;
        this.productSearchBar.value = "";
    }

    /**
     *
     */
    addItem(item: Product): void {
        if (!item) {
            return;
        }

        let index = this.indexOf(item);

        if (index == -1) {
            this.lines.push({ product: item, price: 100, quantity: 1, subtotal: 100 });
        } else {
            let line = this.lines[index];

            line.quantity = line.quantity + 1;
            line.subtotal = line.price * line.quantity;

            this.lines[index] = line;
        }
        
        this.sumTotal();
    }

    /**
     *
     */
    indexOf(item: Product): number {
        for (let i = 0; i < this.lines.length; i++) {
            if (this.lines[i].product.id === item.id) {
                return i;
            }
        }

        return -1;
    }

    /**
     *
     */
    sumTotal(): void {
        let sum: number = 0;
        for (let i = 0; i < this.lines.length; i++) {
            sum = sum + this.lines[i].subtotal;
        }
        
        this.total = sum;
    }

    /**
     *
     */
    addQuantity(item: any): void {
        this.addItem(item.product);
    }

    /**
     *
     */
    removeQuantity(item: any): void {
        let index = this.indexOf(item.product);
        let line = this.lines[index];

        line.quantity = line.quantity - 1;
        line.subtotal = line.price * line.quantity;

        if (line.quantity == 0) {
            this.askIfRemoveItem(item);
            return;
        }

        this.lines[index] = line;
        this.sumTotal();
    }

    /**
     *
     */
    askIfChangePrice(item: any): void {
        this.alertCtrl.create({
            title: "Ingresar",
            message: "Ingrese el precio",
            inputs: [
                {
                    name: "price",
                    type: "number",
                    value: item.price
                }
            ],
            buttons: [
                {
                    text: "Cancelar"
                },
                {
                    text: "Aceptar",
                    handler: data => {
                        this.changePrice(item, data.price);
                    }
                }
            ]
        }).present();
    }

    /**
     *
     */
    isPriceOk(price: number): boolean {
        if (price <= 0) {
            return false;
        }

        return true;
    }

    /**
     *
     */
    changePrice(item: any, price: number): void {
        if (!this.isPriceOk(price)) {
            return;
        }

        let index = this.indexOf(item.product);
        let line = this.lines[index];

        line.price = price;
        line.subtotal = line.price * line.quantity;

        this.lines[index] = line;
        this.sumTotal();
    }

    /**
     *
     */
    askIfRemoveItem(item: Product): void {
        this.alertCtrl.create({
            title: "Confirmar",
            subTitle: "Desea quitar este producto?",
            buttons: [
                {
                    text: "Sí",
                    handler: () => {
                        this.removeItem(item);
                    }
                },
                {
                    text: "No"
                }
            ]
        }).present();
    }

    /**
     *
     */
    removeItem(line: any): void {
        let index = this.lines.indexOf(line);
        this.lines.splice(index, 1);
        this.sumTotal();
    }

    /**
     *
     */
    saveSale(): void {

    }
}
